/*    Copyright (C) 2003 Dave Ackley                         (-*- C++ -*-)

      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation; either version 2 of the License, or
      (at your option) any later version.

      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.

      You should have received a copy of the GNU General Public License
      along with this program; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef OBJECT_H
#define OBJECT_H

// Ackley for CS591 F'2003

#include <assert.h>
#include <vector>
#include "Parms.h"

class World;                    // Forward
class Shark;                    // Forward

class Object {
public:

  Object() ;
  virtual ~Object() ;

  virtual bool Update(World &) = 0;
  virtual bool IsFish() const { return false; }

  size_t GetMass() const { return _mass; }
  size_t AddMass(size_t amt) { _mass += amt; return amt; }
  size_t RemoveMass(size_t amt) {
    if (amt > _mass) amt = _mass;
    _mass -= amt;
    return amt;
  }

  virtual char Code() const = 0;

  void SetXY(size_t x, size_t y) { _x = x; _y = y; }
  void GetXY(size_t & x, size_t & y) { x = _x; y = _y; }

  static void UpdateAll(World & ) ;
  static size_t GetFishCount() { return _fishcount; }
  static size_t GetSharkCount() { return _sharkcount; }
  static size_t GetWander() { return _wandercount; }
  static size_t GetCenter() { return _centercount; }
  static size_t GetTime() { return _time; }

  static Shark * GetRandomShark() ;

protected:

  static size_t _time;
  static size_t _fishcount;
  static size_t _sharkcount;
  static size_t _wandercount;
  static size_t _centercount;

  size_t _x;
  size_t _y;
  size_t _mass;

private:
  typedef vector<Object *> _vOp;
  static _vOp _objects;
  int _idx;

} ;

class Fish : public Object {
public:

  Fish() { ++_fishcount; }
  ~Fish() { assert(_fishcount); --_fishcount; }

  bool IsFish() const { return true; }
  char Code() const { return '.'; }
  bool Update(World &) ;

private:
};

class Shark : public Object {
public:

  Shark() { ++_sharkcount; }    // New shark with random genes
  Shark(const Shark & s) : _genes(s._genes) { ++_sharkcount; } // Offspring shark

  ~Shark() { assert(_sharkcount); --_sharkcount; }

  char Code() const { return '*'; }
  bool Update(World &) ;

  // Genome is (currently) 32 bits, arranged as follows:
  // 33222222222211111111110000000000
  // 10987654321098765432109876543210
  // xxxxxMV4MV3MV2MV1MV0-TOKID-BREED

  struct Genome {
    enum {                      // Indices into code
      BREED = 0,                // Index of mass to try splitting at
      BREEDBITS = 6,            // Six bits for breed, so 0..63
      TOKID = BREED+BREEDBITS,  // Index of mass to pass to kid
      TOKIDBITS = 6,            // 0..63 for that

      MOVE = TOKID+TOKIDBITS,   // Index of beginning of move codes
      MOVEBITS = 3,             // 0..7 per move code
      MOVECOUNT = 5,            // Number of move codes

      PSEUDO = MOVE+MOVEBITS*MOVECOUNT, // Unused gene bits
      PSEUDOBITS = 32-PSEUDO
    };

    size_t GetBreed() const {
      return (code>>BREED)&((1<<BREEDBITS)-1);
    }
    size_t GetToKid() const {
      return (code>>TOKID)&((1<<TOKIDBITS)-1);
    }
    size_t GetMoveCode(size_t which) const {
      return (code>>(MOVE+MOVEBITS*which))&((1<<MOVEBITS)-1);
    }
    size_t GetPseudo() const {
      return (code>>PSEUDO)&((1<<PSEUDOBITS)-1);
    }

    Genome() ;                 // Get set of random genes
    Genome(const Genome &) ;  // Copy genes with mutation

    size_t code;
  };

  Genome GetGenome() const { return _genes; }

private:

  Genome _genes;
};

#endif
