/*    Copyright (C) 2003 Dave Ackley                         (-*- C++ -*-)

      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation; either version 2 of the License, or
      (at your option) any later version.

      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.

      You should have received a copy of the GNU General Public License
      along with this program; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using namespace std;

#include "Parms.h"
#include <string>
#include <iostream>

Parms * Parms::_parmptr = 0;

// DEFAULT PARAMETER VALUES SET HERE
Parms::Parms()
  : _flags(0),
    _seed(0),
    _width(32),
    _height(32),
    _initfish(200),
    _initshark(100),
    _fishbite(3),
    _fishfull(9),
    _sharkburn(1),
    _mutationodds(100)
{}

Parms::~Parms()
{
  if (_parmptr == this)
    _parmptr = 0;
}

static struct {
  string longname;
  string shortname;
  void (Parms::*funcptr)(size_t);
  size_t flagindex;
  size_t flagvalue;
  string help;
} args[] = {
  {"--flags", "-f", &Parms::SetFlags, 0, 0, "Set the flags to NUM"},
  {"--seed", "-s", &Parms::SetSeed, 0, 0, "Set random number seed to NUM"},
  {"--width", "-w", &Parms::SetWidth, 0, 0, "Set world width to NUM"},
  {"--height", "-h", &Parms::SetHeight, 0, 0, "Set world height to NUM"},
  {"--initfish", "-if", &Parms::SetInitFish, 0, 0, "Start with NUM fish"},
  {"--initshark", "-is", &Parms::SetInitShark, 0, 0, "Start with NUM sharks"},
  {"--fishbite", "-fb", &Parms::SetFishBite, 0, 0, "Fish eat NUM weight every day"},
  {"--fishfull", "-ff", &Parms::SetFishFull, 0, 0, "Fish weighing NUM are big enough to spawn"},
  {"--sharkburn", "-sb", &Parms::SetSharkBurn, 0, 0, "Shark burn up NUM weight per move scan"},
  {"--mutationodds", "-mo", &Parms::SetMutationOdds, 0, 0, "Shark genes mutate 1 in NUM per bit"},

  {"--displaygrid", "-dg", 0, Parms::DISPLAYGRID, 1, "Display grid each day"},
  {"--displayclear", "-dc", 0, Parms::DISPLAYCLEAR, 1, "Clear screen each day"},
  {"--dontstop", "-ds", 0, Parms::DONTSTOP, 1, "Don't stop on extinction"},
  {"--displayrandomshark", "-drs", 0, Parms::DISPLAYRANDOMSHARK, 1, "Print a randomly-picked shark"},

  {"--help", "-?", 0, Parms::PRINTHELP, 1, "Print help and exit"}
};

bool Parms::ProcessArgs(int argc,char ** argv)
{
  assert(argc>0);

  --argc; ++argv;

  while (argc) {
    string cmd = argv[0];
    --argc; ++argv;

    bool found = false;
    for (size_t i = 0; i < sizeof(args)/sizeof(args[0]); ++i) {
      if (cmd==args[i].longname || cmd==args[i].shortname) {
        found = true;

        if (args[i].funcptr) {

          if (argc==0) {
            cerr
              << "Switch: '" << cmd
              << "' needs argument, use -? for help" << endl;
            return false;
          }
          size_t val = atoi(argv[0]);
          --argc; ++argv;

          (this->*args[i].funcptr)(val);
        } else {

          size_t bit = args[i].flagvalue;
          size_t pos = args[i].flagindex;
          size_t mask = 1<<pos;
          size_t val = bit<<pos;
          SetFlags(GetFlags()&~mask|val);

        }
        break;
      }
    }
    if (!found) {
      cerr << "Unknown switch: '" << cmd << "', use -? for help" << endl;
      return false;
    }
  }

  if (GetFlag(PRINTHELP)) {
    cerr << "Switches are:" << endl;
    for (size_t i = 0; i < sizeof(args)/sizeof(args[0]); ++i) {

      string argt;
      if (args[i].funcptr) argt = " NUM"; else argt = "    ";
      
      size_t longlen = args[i].longname.size();
      cerr << "  " << args[i].longname << argt;
      while (longlen++<15) cerr << ' ';

      cerr << "  " << args[i].shortname << argt;
      size_t shortlen = args[i].shortname.size();
      while (shortlen++<5) cerr << ' ';

      cerr << " " << args[i].help << endl;
    }
    exit(0);
  }

  return true;
}
