/*    Copyright (C) 2003 Dave Ackley                         (-*- C++ -*-)

      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation; either version 2 of the License, or
      (at your option) any later version.

      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.

      You should have received a copy of the GNU General Public License
      along with this program; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef PARMS_H
#define PARMS_H

// Ackley for CS591 F'2003

#include <stddef.h>             /* for size_t */ 
#include <assert.h>             /* for assert */ 
#include <stdlib.h>             /* for random */ 

typedef unsigned char u_char;

class Parms {
public:
  Parms() ;
  ~Parms() ;

  // Initialize a parameter block from cmd line args
  bool ProcessArgs(int argc,char ** argv) ;

  // Get and set the current global parameter block
  static Parms & Global() { assert(_parmptr); return *_parmptr; }
  void SetGlobal() { _parmptr = this; }

  // Settable flags controlling model behavior
  enum {
    DISPLAYGRID,                // Print the whole grid each step
    DISPLAYCLEAR,               // Clear terminal screen and pause per step
    DONTSTOP,                   // Keep going even after something is extinct
    DISPLAYRANDOMSHARK,         // Print the dna of a random shark once a day
    
    PRINTHELP                   // Print help and quit
  };

  // Accessors
  bool GetFlag(size_t flagidx) const { return _flags&(1<<flagidx); }
  size_t GetFlags() const { return _flags; }
  size_t GetSeed() const { return _seed; }
  size_t GetWidth() const { return _width; }
  size_t GetHeight() const { return _height; }
  size_t GetInitFish() const { return _initfish; }
  size_t GetInitShark() const { return _initshark; }
  size_t GetFishBite() const { return _fishbite; }
  size_t GetFishFull() const { return _fishfull; }
  size_t GetSharkBurn() const { return _sharkburn; }

  size_t GetMutationOdds() const { return _mutationodds; }

  // Updaters
  void SetFlags(size_t flags) { _flags = flags; }
  void SetSeed(size_t seed) { _seed = seed; }
  void SetWidth(size_t width) { _width = width; }
  void SetHeight(size_t height) { _height = height; }
  void SetInitFish(size_t initfish) { _initfish = initfish; }
  void SetInitShark(size_t initshark) { _initshark = initshark; }
  void SetFishBite(size_t fb) {assert(fb<128); _fishbite = fb;}
  void SetFishFull(size_t ff) { _fishfull = ff;}
  void SetSharkBurn(size_t sb) {_sharkburn = sb; }

  void SetMutationOdds(size_t mo) { _mutationodds = mo; }
  
private:

  static Parms * _parmptr;

  size_t _flags;
  size_t _seed;
  size_t _width;
  size_t _height;
  size_t _initfish;
  size_t _initshark;
  size_t _fishbite;
  size_t _fishfull;
  size_t _sharkburn;
  size_t _mutationodds;
} ;

#endif
