# Copyright (C) 2003 Dave Ackley.  All rights reserved.
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

source init0.tcl

Proc OnLoad {} {
    LoadInit
    BuildInterface
}

Proc LoadInit {} {
    global D A
    
    # Flush any old stuff
    catch {unset D}
    catch {unset A}
    
    # Widgets and variables
    set D(top) .foo
    set D(sgene) $D(top).sgene
    set D(map) $D(top).map
    set D(cmdf) $D(top).cmdf
    set D(runb) $D(cmdf).runb
    set D(resetb) $D(cmdf).resetb

    set D(image) mapimg

    # GUI parameters
    set D(msdelay) 1

    # Wator command line args and defaults:
    array set A {
        displaygrid ""
        displayrandomshark ""
        width 50
        height 50
        seed 0
        initfish 600
        initshark 600
        fishbite 1
        fishfull 6
        sharkburn 1
        mutationodds 100
    }
}

Proc BuildInterface {} {
    global D
    catch {destroy $D(top)}

    frame $D(top)
    pack $D(top) \
        -expand 1 \
        -fill both

    set D(randomsgene) ""
    label $D(sgene) \
        -textvariable D(randomsgene) \
        -font courier
    

    pack $D(sgene) \
        -side top \
        -anchor w \
        -expand 1

    canvas $D(map)
    pack $D(map) \
        -expand 1 \
        -fill both

    frame $D(cmdf)
    pack $D(cmdf) \
        -side bottom \
        -expand 1 \
        -fill x

    button $D(resetb) \
        -text "Reset" \
        -command "ResetWator"
    pack $D(resetb) \
        -side left

    set D(running) 0
    checkbutton $D(runb) \
        -command "RunWator" \
        -indicatoron 0 \
        -text "Run" \
        -variable D(running) \
        -state disabled
    pack $D(runb) \
        -side left \
        -fill y

}

Proc ResetWator {} {
    global D A

    ResetImage

    set cmd "./Wator"
    foreach i [array names A] {
        append cmd " --$i " $A($i)
    }
    catch {close $D(proc)}
    set D(proc) [open "|$cmd" "r"]
    set D(buffer) {}
    
    $D(runb) config -state normal

    ClearStats

    RunWator
}

Proc ClearStats {} {
    global D
    set D(avgbreed) 0.0
    set D(avgtokid) 0.0
}

Proc ResetImage {} {
    global D A

    catch {$D(map) delete all}
    catch {image delete $D(image)}

    set D(pixperwidth) 8
    set D(pixperheight) 6
    image create photo $D(image) \
        -width [expr $D(pixperwidth)*$A(width)] \
        -height [expr $D(pixperheight)*$A(height)]

    $D(map) create image 0 0 \
        -anchor nw \
        -image $D(image)
}

Proc RunWator {} {
    global D A
    after cancel RunWator
    if {$D(running)} {
        after $D(msdelay) RunWator
    }
    set counts [gets $D(proc)]
    if {$counts == ""} {
        set D(running) 0
        close $D(proc)
        $D(runb) config -state disabled
        return
    }
    DisplayCounts [lindex $counts 0] [lindex $counts 1] [lindex $counts 2] [lindex $counts 3] [lindex $counts 4] [lindex $counts 5]
    for {set i 0} {$i < $A(height)} {incr i} {
        DisplayRow $i [gets $D(proc)]
    }
    if {[info exists A(displayrandomshark)]} {
        DisplaySgene [gets $D(proc)]
    }
    update idletasks
}

Proc DisplayCounts {time mass fish sharks wander center} {
    global D
    puts "$time; $mass; $fish; $sharks; $wander; $center"
}

Proc DisplaySgene {data} {
    global D
    set codes {C N S E W X X X} 
    set t ""
    for {set i 1} {$i < 6} {incr i} {
        append t [lindex $codes [lindex $data $i]]
    }
    set breed [lindex $data 6]
    set tokid [lindex $data 7]
    set pseudo [lindex $data 8]

    set D(avgbreed) [expr .95*$D(avgbreed) + .05*$breed]
    set D(avgtokid) [expr .95*$D(avgtokid) + .05*$tokid]

    append t [format " %02d %02d %02x (%5.2f %5.2f)" \
              $breed $tokid $pseudo $D(avgbreed) $D(avgtokid)]

    set D(randomsgene) $t
}

Proc DisplayRow {row data} {
    global D A
    set dx $D(pixperwidth)
    set dy $D(pixperheight)
    set sy [expr $row*$dy]
    for {set i 0} {$i < $A(width)} {incr i} {
        set ch [string index $data $i]
        set col "\#1010ff"
        if {$ch == "*"} {
            set col "\#ff1010"
        } elseif {$ch == "."} {
            set col "\#10ff10"
        }
        set sx [expr $i*$dx]
        $D(image) put $col -to \
            [expr round($sx)] [expr round($sy)] \
            [expr round($sx+$dx)] [expr round($sy+$dy)]
    }
}

OnLoad
