/*    Copyright (C) 2003 Dave Ackley                         (-*- C++ -*-)

      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation; either version 2 of the License, or
      (at your option) any later version.

      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.

      You should have received a copy of the GNU General Public License
      along with this program; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using namespace std;

#include "World.h"
#include "Object.h"
#include "Parms.h"

World::World() : _mass(0), _width(0), _height(0)
{
  for (size_t i = 0; i<3; ++i) _counts[i] = 0;
}

World::~World()
{
  _Flush();
}

void World::Resize(size_t wid, size_t hei)
{
  _Flush();

  _width = wid;
  _height = hei;

  _map.resize(_width);

  for (size_t x = 0; x < _width; ++x) {
    _map[x].resize(_height);

    for (size_t y = 0; y < _height; ++y) {
      _map[x][y] = 0;
    }
  }
}

void World::_Flush()
{
  for (size_t x = 0; x < _width; ++x) {
    for (size_t y = 0; y < _height; ++y) {
      if (Object *& fp = _map[x][y]) {
        delete fp;
        fp = 0;
      }
    }
  }
}


void World::_Wrap(size_t & x, size_t & y, Direction dir)
{
  switch (dir) {
  case CENTER: break;

  case NORTH:  if (y == 0) y = _height-1; else --y; break;
  case SOUTH:  if (y == _height-1) y = 0; else ++y; break;

  case WEST:   if (x == 0) x = _width-1; else --x; break;
  case EAST:   if (x == _width-1) x = 0; else ++x; break;
  }
}

Object * World::Get(size_t x, size_t y) const
{
  assert(x<_width && y<_height);
  return _map[x][y];

}

void World::Put(size_t x, size_t y, Object * o)
{
  assert(x<_width && y<_height);

  _map[x][y] = o;

  if (o)
    o->SetXY(x,y);
}

void World::Dump(ostream &o) const
{
  Parms & p = Parms::Global();

  if (p.GetFlag(Parms::DISPLAYCLEAR))
    system("clear");
  o
    << Object::GetTime() << ' '
    << GetMass() << ' '
    << Object::GetFishCount() << ' '
    << Object::GetSharkCount() << ' '
    << Object::GetWander() << ' '
    << Object::GetCenter() << endl;

  if (p.GetFlag(Parms::DISPLAYGRID)) {
    for (size_t y = 0; y < _height; ++y) {
      for (size_t x = 0; x < _width; ++x) {
        if (Object * ob = _map[x][y]) 
          o << ob->Code();
        else
          o << ' ';
      }
      o << endl;
    }
  }

  if (p.GetFlag(Parms::DISPLAYRANDOMSHARK)) {
    Shark * s = Object::GetRandomShark() ;
    if (!s) o << "x" << endl;   // flag no shark
    else {
      Shark::Genome g = s->GetGenome();
      o << 's';
      for (size_t idx = 0; idx<Shark::Genome::MOVECOUNT; ++idx) {
        o << ' ' << g.GetMoveCode(idx);
      }
      o
        << ' ' << g.GetBreed()
        << ' ' << g.GetToKid()
        << ' ' << g.GetPseudo()
        << endl;
    }
  }

  if (p.GetFlag(Parms::DISPLAYCLEAR))
    system("sleep .01");
}

#if 1
void World::GetXY(size_t & x, size_t & y, size_t dir)
{
  switch (dir) {
  case CENTER: break;

  case NORTH:  if (y == 0) y = _height-1; else --y; break;
  case SOUTH:  if (y == _height-1) y = 0; else ++y; break;

  case WEST:   if (x == 0) x = _width-1; else --x; break;
  case EAST:   if (x == _width-1) x = 0; else ++x; break;
  }
}
#else
void World::GetXY(size_t & x, size_t & y, size_t dir)
{
  switch (dir) {
  case CENTER: break;

  case NORTH:  if (y == 0) { y = _height-1; x = (_width-1)-x; } else --y; break;
  case SOUTH:  if (y == _height-1) { y = 0; x = (_width-1)-x; } else ++y; break;

  case WEST:   if (x == 0) x = _width-1; else --x; break;
  case EAST:   if (x == _width-1) x = 0; else ++x; break;
  }
}
#endif
